@@ -6,7 +6,7 @@ __all__ = ['Controller', 'absolute_url',
     'Root', 'RootController', 'url', 'validate']
 
 from functools import wraps
-from inspect import isclass
+import inspect
 import logging
 import urllib.request, urllib.parse, urllib.error
 import urllib.parse
@@ -81,6 +81,10 @@ def validate(form=None, validators=None,
         else:
             init_form = lambda self: form
 
+        # Extract defaults from the function signature at definition to
+        # supply values for missing arguments during validation
+        default_values = tg_util.get_kw_defaults(func)
+
         @wraps_exposed(func)
         def validate(*args, **kw):
             # do not validate a second time if already validated
@@ -109,8 +113,9 @@ def validate(form=None, validators=None,
                 if isinstance(validators, dict):
                     for field, validator in validators.items():
                         try:
-                            kw[field] = validator.to_python(
-                                kw.get(field, None), state)
+                            raw_value = \
+                                kw.get(field, default_values.get(field, None))
+                            kw[field] = validator.to_python(raw_value, state)
                         except Invalid as error:
                             errors[field] = error
                 else:
@@ -128,6 +133,7 @@ def validate(form=None, validators=None,
                 kw = errorhandling.dispatch_failsafe(failsafe_schema,
                                             failsafe_values, errors, func, kw)
             args, kw = tg_util.from_kw(func, args, kw)
+
             return errorhandling.run_with_errors(errors, func, *args, **kw)
 
         return validate
@@ -195,7 +201,7 @@ def _execute_func(func, template, format, content_type, fragment, options,
         tg_kw = {}
 
     # remove excess parameters
-    args, kw = tg_util.adapt_call(func, args, kw)
+    args, kw = tg_util.adapt_call(inspect.unwrap(func), args, kw)
 
     # add special parameters again
     kw.update(tg_kw)
@@ -232,6 +238,10 @@ def _process_output(output, template, format, content_type, fragment=False,
             pass
 
         return
+    elif isinstance(output, types.GeneratorType):  # let cp handle streaming
+        if content_type:
+            response.headers['Content-Type'] = content_type
+        return output
     elif response.body:
         # If response.body is set, the view handler has already set a response
         # (e.g. a file), so don't process further
@@ -353,35 +363,6 @@ def flash(message):
     response.cookie['tg_flash']['path'] = '/'
 
 
-def _get_flash():
-    '''Retrieve the flash message (if one is set), clearing the message.'''
-    request_cookie = request.cookie
-    response_cookie = response.cookie
-
-    def clearcookie():
-        response_cookie['tg_flash'] = ''
-        response_cookie['tg_flash']['expires'] = 0
-        response_cookie['tg_flash']['path'] = '/'
-
-    if 'tg_flash' in response_cookie:
-        message = response_cookie['tg_flash'].value
-        response_cookie.pop('tg_flash')
-        if 'tg_flash' in request_cookie:
-            # New flash overrided old one sitting in cookie. Clear that old cookie.
-            clearcookie()
-    elif 'tg_flash' in request_cookie:
-        message = request_cookie.value_decode(request_cookie['tg_flash'].value)[0]
-        if 'tg_flash' not in response_cookie:
-            clearcookie()
-    else:
-        message = None
-
-    # if message:
-    #     message = str(message, 'utf-8')
-
-    return message
-
-
 class Controller(object):
     """Base class for a web application's controller.
 
