@@ -9,8 +9,8 @@ from past.builtins import basestring
 from builtins import object
 _all__ = ['Bunch', 'DictObj', 'DictWrapper', 'Enum', 'setlike',
    'get_package_name', 'get_model', 'load_project_config',
-   'ensure_sequence', 'has_arg', 'to_kw', 'from_kw', 'adapt_call',
-   'call_on_stack', 'remove_keys', 'arg_index',
+   'ensure_sequence', 'has_arg', 'get_kw_defaults', 'to_kw', 'from_kw',
+   'adapt_call', 'call_on_stack', 'remove_keys', 'arg_index',
    'inject_arg', 'inject_args', 'add_tg_args', 'bind_args',
    'recursive_update', 'combine_contexts',
    'request_available', 'flatten_sequence', 'load_class',
@@ -29,7 +29,8 @@ import warnings
 import html.entities
 import socket
 import struct
-from inspect import getargspec, getargvalues
+import inspect
+from inspect import getargspec, getargvalues, getfullargspec
 from itertools import islice, chain
 from http.cookies import _quote as quote_cookie, _unquote as unquote_cookie
 
@@ -263,23 +264,52 @@ def ensure_sequence(obj):
         return [obj]
 
 
+def get_kw_defaults(func):
+    """Produce a mapping of a callable's keyword parameters to defaults.
+
+    Given a callable, return a mapping from the name of each keyword parameter
+    to the corresponding default value defined on that parameter.
+
+    Returns a dict, possibly empty.
+
+    """
+
+    sig = inspect.signature(func)
+
+    defaults = {}
+
+    for parameter_name in sig.parameters:
+        if sig.parameters[parameter_name].default != inspect._empty:
+            defaults[parameter_name] = sig.parameters[parameter_name].default
+
+    return defaults
+
+
 def to_kw(func, args, kw, start=0):
     """Convert all applicable arguments to keyword arguments."""
-    argnames, defaults = getargspec(func)[::3]
-    defaults = ensure_sequence(defaults)
-    kv_pairs = zip(
-        islice(argnames, start, len(argnames) - len(defaults)), args)
-    for k, v in kv_pairs:
-        kw[k] = v
-    return args[len(argnames)-len(defaults)-start:], kw
+
+    spec = getfullargspec(func)
+    args = list(args)
+    kw = kw.copy()
+
+    for parameter_name in spec.args:
+        if not args or parameter_name in kw:
+            break
+        else:
+            v = args.pop(0)
+            kw[parameter_name] = v
+
+    return args, kw
 
 
 def from_kw(func, args, kw, start=0):
     """Extract named positional arguments from keyword arguments."""
-    argnames, defaults = getargspec(func)[::3]
+
+    argnames, varargs, varkw, defaults = getfullargspec(func)[:4]
+
     defaults = ensure_sequence(defaults)
     newargs = [kw.pop(name) for name in islice(argnames, start,
-        len(argnames) - len(defaults)) if name in kw]
+               len(argnames) - len(defaults)) if name in kw]
     newargs.extend(args)
     return newargs, kw
 
