@@ -153,12 +153,13 @@ def render(info, template=None, format=None, headers=None, fragment=False,
         template = format == 'json' and 'json' or info.pop(
             'tg_template', template)
 
-        # FIXME (Py3K)
-        tg_flash = None # _get_flash()
-        if tg_flash:
-            info['tg_flash'] = tg_flash
-        elif config.get('tg.empty_flash', True):
-            info['tg_flash'] = None
+        if 'tg_flash' not in info:
+            tg_flash = _get_flash()
+            if tg_flash:
+                info['tg_flash'] = tg_flash
+
+        if config.get('tg.empty_flash', True):
+            info.setdefault('tg_flash', None)
 
     engine, template, engine_name = _choose_engine(template)
 
@@ -464,6 +465,38 @@ def stdvars():
     return root_vars
 
 
+def _get_flash():
+    """Retrieve the flash message (if one is set), clearing the message."""
+
+    request, response = cherrypy.request, cherrypy.response
+
+    request_cookie = request.cookie
+    response_cookie = response.cookie
+
+    def clearcookie():
+        response_cookie['tg_flash'] = ''
+        response_cookie['tg_flash']['expires'] = 0
+        response_cookie['tg_flash']['path'] = '/'
+
+    if 'tg_flash' in response_cookie:
+        message = response_cookie['tg_flash'].value
+        response_cookie.pop('tg_flash')
+
+        # If there was an old tg_flash value in the request cookie, clear it;
+        # its value is overridden by the new flash in the response
+        if 'tg_flash' in request_cookie:
+            clearcookie()
+    elif 'tg_flash' in request_cookie:
+        message = \
+            request_cookie.value_decode(request_cookie['tg_flash'].value)[0]
+        if 'tg_flash' not in response_cookie:
+            clearcookie()
+    else:
+        message = None
+
+    return message
+
+
 def _get_plugin_options(plugin_name, defaults=None):
     """Return all options from global config where the first part of the config
     setting name matches the start of plugin_name.
