@@ -21,6 +21,9 @@ class DefaultJSONRendererPlugin:
     `turbogears.view.engines`. If no other plugin defines a 'json' engine,
     `view.engines['json']` will also refer to this renderer.
 
+    If the incoming data is `bytes`, this is a pass-thru (we assume
+    the incoming data may already be encoded for delivery).
+
     Utilizes simplejson and a custom `default` function to automatically
     serialize
 
@@ -39,6 +42,11 @@ class DefaultJSONRendererPlugin:
         pass
 
     def render(self, info, format=None, fragment=False, template=None):
+        # A bytes() object is most likely something that has already been
+        # JSON-encoded.
+        if isinstance(info, bytes):
+            return info
+
         if isinstance(info, dict):
             info = {
                 k:v for k, v in info.items()
